//
//  SCSImpulseChartView.swift
//  SciChartSwiftDemo
//
//  Created by Hrybenuik Mykola on 9/15/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import UIKit
import SciChart

class SCSImpulseChartView: SCSBaseChartView {
    
    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxes()
        addSeries()
    }
    
    // MARK: Private Functions
    
    private func addAxes() {
        
        let axisStyle = generateDefaultAxisStyle()
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisXId), isXAxis: true)
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId), isXAxis: false)
        addDefaultModifiers()
        
    }
    
    private func addSeries() {
        
        let ellipsePointMarker = SCIEllipsePointMarker()
        ellipsePointMarker.drawBorder = false
        ellipsePointMarker.fillBrush = SCIBrushSolid(color: UIColor.blueColor())
        ellipsePointMarker.height = 10
        ellipsePointMarker.width = 10
        
        let dataSeries = SCSDataManager.getDampedSinewave(1.0,
                                                          phase: 0.0,
                                                          dampingFactor: 0.05,
                                                          pointCount: 50,
                                                          freq: 5)
     
        let impulseRenderSeries = SCIFastImpulseRenderableSeries()
        impulseRenderSeries.dataSeries = dataSeries
        impulseRenderSeries.style.pointMarker = ellipsePointMarker;
        impulseRenderSeries.xAxisId = axisXId
        impulseRenderSeries.yAxisId = axisYId
        impulseRenderSeries.style.linePen = SCIPenSolid(color: UIColor.blueColor(), width: 0.7)
        chartSurface.attachRenderableSeries(impulseRenderSeries)
        
        chartSurface.invalidateElement()
        
    }
    
}
