//
//  SCSScatterSeriesChartView.swift
//  SciChartSwiftDemo
//
//  Created by Mykola Hrybeniuk on 6/3/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import Foundation
import SciChart

class SCSScatterSeriesChartView: SCSBaseChartView {
    
    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxis()
        addDefaultModifiers()
        addDataSeries()
    }
    
    override func addDefaultModifiers() {
        super.addDefaultModifiers()
        
        let cursor = SCICursorModifier()
        cursor.style.hitTestMode = .Point
        cursor.modifierName = "CursorModifier"
        cursor.style.hitTestMode = .Point
        cursor.style.colorMode = SCITooltipColorMode.SeriesColorToDataView;
        cursor.style.tooltipSize = CGSizeMake(200, CGFloat.NaN)
        
        let group = chartSurface.chartModifier as! SCIModifierGroup
        group .removeAt(group.itemCount()-1);
        group .addItem(cursor)
    }
    
    // MARK: Private Methods
    
    private func addAxis() {
        
        let axisStyle = generateDefaultAxisStyle()
        
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultDateTimeAxis(withAxisStyle: axisStyle, axisId: axisXId), isXAxis: true)
        
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId),
                                isXAxis: false)
    }
    

    private func addDataSeries() {
        
        chartSurface.attachRenderableSeries(getScatterRenderableSeries(withDetalization: 3,
            colorCode: 0xFF01ebff,
            negative: false))
        chartSurface.attachRenderableSeries(getScatterRenderableSeries(withDetalization: 6,
            colorCode: 0xFF00a3ff,
            negative: false))
        
        chartSurface.attachRenderableSeries(getScatterRenderableSeries(withDetalization: 3,
            colorCode: 0xFF0165ff,
            negative: true))
        chartSurface.attachRenderableSeries(getScatterRenderableSeries(withDetalization: 6,
            colorCode: 0xFF00a3ff,
            negative: true))
        
        chartSurface.invalidateElement()
        
    }
    
    private func getScatterRenderableSeries(withDetalization pointMarker: Int32, colorCode: UInt32, negative: Bool) -> SCIXyScatterRenderableSeries {
        
        
        let dataSeries = SCIXyDataSeries(XType: .DateTime, YType: .Float)
        putDataInto(dataSeries, belowZeroY: negative)
        
        dataSeries.seriesName = (pointMarker == 6) ?
            (negative ? "Negative Hex" : "Positive Hex") :
            (negative ? "Negative" : "Positive")
        dataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        let scatterRenderableSeries = SCIXyScatterRenderableSeries()
        scatterRenderableSeries.xAxisId = axisXId
        scatterRenderableSeries.yAxisId = axisYId
        scatterRenderableSeries.dataSeries = dataSeries
        
        let ellipse = SCIEllipsePointMarker()
        ellipse.drawBorder = true
        ellipse.fillBrush = SCIBrushSolid(colorCode: colorCode)
        ellipse.borderPen = SCIPenSolid(colorCode: 0xfffffff, width: 0.1)
        ellipse.detalization = pointMarker
        ellipse.height = 6.0
        ellipse.width = 6.0
        
        scatterRenderableSeries.style.pointMarker = ellipse
        
        return scatterRenderableSeries
    }
    
    private func putDataInto(dataSeries: SCIXyDataSeries, belowZeroY: Bool) {
        
        var i : UInt32 = 0
        while i < 200 {
            
            let x = i
            let time = (i < 100) ? arc4random_uniform(x+10) : arc4random_uniform(200-x+10)
            let y = time*time*time
            
            if belowZeroY {
                dataSeries.appendX(SCIGeneric(Double(x)), y: SCIGeneric(-Int32(y)))
            }
            else {
                dataSeries.appendX(SCIGeneric(Double(x)), y: SCIGeneric(y))
            }
            
            i = i + 1
        }
        
    }
    
}
